#ifndef LIBRARIES_LIBUSB_H
#define LIBRARIES_LIBUSB_H

/*------------------------------------------------------------------------*/

#ifndef  DOS_DOS_H
#include <dos/dos.h>
#endif

/*------------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------*/

#ifdef __GNUC__
    #ifdef __PPC__
        #pragma pack(2)
    #endif
#elif defined(__VBCC__)
    #pragma amiga-align
#endif

/*------------------------------------------------------------------------*/

#define USB_LE16_TO_CPU(x) do { x = ((x & 0xff) << 8) | ((x & 0xff00) >> 8); } while(0)

/* Some quick and generic macros for the simple kind of lists we use */
#define LIST_ADD( begin, ent ) \
    do { \
        if (begin) { \
            ent->next = begin; \
            ent->next->prev = ent; \
        } else \
            ent->next = NULL; \
        ent->prev = NULL; \
        begin = ent; \
    } while(0)

#define LIST_DEL( begin, ent ) \
    do { \
        if (ent->prev) \
            ent->prev->next = ent->next; \
        else \
            begin = ent->next; \
        if (ent->next) \
            ent->next->prev = ent->prev; \
        ent->prev = NULL; \
        ent->next = NULL; \
    } while (0)

#define USB_ERROR(x) \
    do { \
        libBase->lib_usb_error_type = USB_ERROR_TYPE_ERRNO; \
        libBase->lib_usb_error_errno = x; \
        RETURN( x ); \
        return( x ); \
    } while (0)

/*------------------------------------------------------------------------*/

struct LibusbBase
{
    APTR                    lib_Succ;        // Pointer to next (successor)
    APTR                    lib_Pred;        // Pointer to previous (predecessor)
    uint8                   lib_Type;
    int8                    lib_Pri;         // Priority, for sorting
    STRPTR                  lib_Name;        // ID string, null terminated
    uint8                   lib_Flags;       // see below
    uint8                   lib_ABIVersion;  // ABI exported by library
    uint16                  lib_NegSize;     // number of bytes before LIB
    uint16                  lib_PosSize;     // number of bytes after LIB
    uint16                  lib_Version;     // major
    uint16                  lib_Revision;    // minor
    STRPTR                  lib_IdString;    // ASCII identification
    uint32                  lib_Sum;         // the system-calculated checksum
    uint16                  lib_OpenCnt;     // number of current opens
	uint16					lib_Pad;
    APTR                    lib_SegmentList;

	struct Library *		lib_DOSBase;
	struct DOSIFace *		lib_IDOS;

	struct Library *		lib_ExecBase;
	struct ExecIFace *		lib_IExec;
	
	struct Library *		lib_UtilityBase;
	struct UtilityIFace *	lib_IUtility;
    
    struct usb_bus *        lib_usb_busses;
    uint32                  lib_usb_debug;
    uint32                  lib_usb_error_errno;
    uint32                  lib_usb_error_type;

	BPTR					lib_stderr;
	TEXT					lib_EnvBuffer[256];

	// Debug
	uint32					debug_Level;
	uint32					debug_Indent_level;
	uint32					debug_Program_name_len;
	TEXT					debug_Program_name[64];
};

struct LibusbAmigaData
{
    struct UsbRawInterface *lad_RawInterface;
    struct UsbInterface *   lad_Interface;
    struct UsbEndPoint *    lad_EndPoints[32];
};

//------------------------------------

/*
 * USB spec information
 *
 * This is all stuff grabbed from various USB specs and is pretty much
 * not subject to change
 */

#define DESC_HEADER_LENGTH          2
#define DEVICE_DESC_LENGTH          18
#define CONFIG_DESC_LENGTH          9
#define INTERFACE_DESC_LENGTH       9
#define ENDPOINT_DESC_LENGTH        7
#define ENDPOINT_AUDIO_DESC_LENGTH  9

/*
 * Device and/or Interface Class codes
 */
#define USB_CLASS_PER_INTERFACE     0   /* for DeviceClass */
#define USB_CLASS_AUDIO             1
#define USB_CLASS_COMM              2
#define USB_CLASS_HID               3
#define USB_CLASS_PTP               6
#define USB_CLASS_PRINTER           7
#define USB_CLASS_MASS_STORAGE      8
#define USB_CLASS_HUB               9
#define USB_CLASS_DATA              10
#define USB_CLASS_VENDOR_SPEC       255

/*
 * Descriptor types
 */
#define USB_DT_DEVICE               0x01
#define USB_DT_CONFIG               0x02
#define USB_DT_STRING               0x03
#define USB_DT_INTERFACE            0x04
#define USB_DT_ENDPOINT             0x05

#define USB_DT_HID                  0x21
#define USB_DT_REPORT               0x22
#define USB_DT_PHYSICAL             0x23
#define USB_DT_HUB                  0x29

/*
 * Descriptor sizes per descriptor type
 */
#define USB_DT_DEVICE_SIZE          18
#define USB_DT_CONFIG_SIZE          9
#define USB_DT_INTERFACE_SIZE       9
#define USB_DT_ENDPOINT_SIZE        7
#define USB_DT_ENDPOINT_AUDIO_SIZE  9   /* Audio extension */
#define USB_DT_HUB_NONVAR_SIZE      7

/*
 * Standard requests
 */
#define USB_REQ_GET_STATUS          0x00
#define USB_REQ_CLEAR_FEATURE       0x01
/* 0x02 is reserved */
#define USB_REQ_SET_FEATURE         0x03
/* 0x04 is reserved */
#define USB_REQ_SET_ADDRESS         0x05
#define USB_REQ_GET_DESCRIPTOR      0x06
#define USB_REQ_SET_DESCRIPTOR      0x07
#define USB_REQ_GET_CONFIGURATION   0x08
#define USB_REQ_SET_CONFIGURATION   0x09
#define USB_REQ_GET_INTERFACE       0x0A
#define USB_REQ_SET_INTERFACE       0x0B
#define USB_REQ_SYNCH_FRAME         0x0C

#define USB_TYPE_STANDARD           (0x00 << 5)
#define USB_TYPE_CLASS              (0x01 << 5)
#define USB_TYPE_VENDOR             (0x02 << 5)
#define USB_TYPE_RESERVED           (0x03 << 5)

#define USB_RECIP_DEVICE            0x00
#define USB_RECIP_INTERFACE         0x01
#define USB_RECIP_ENDPOINT          0x02
#define USB_RECIP_OTHER             0x03

#define USB_ENDPOINT_ADDRESS_MASK   0x0f    /* in bEndpointAddress */
#define USB_ENDPOINT_DIR_MASK       0x80

#define USB_ENDPOINT_TYPE_MASK          0x03    /* in bmAttributes */
#define USB_ENDPOINT_TYPE_CONTROL       0
#define USB_ENDPOINT_TYPE_ISOCHRONOUS   1
#define USB_ENDPOINT_TYPE_BULK          2
#define USB_ENDPOINT_TYPE_INTERRUPT     3

/*
 * Various libusb API related stuff
 */

#define USB_ENDPOINT_IN             0x80
#define USB_ENDPOINT_OUT            0x00

/* Error codes */
#define USB_ERROR_BEGIN             500000

//------------------------------------

#define PATH_MAX                    511
#define usb_busses                  ILibusb->usb_get_busses()

typedef enum {
  USB_ERROR_TYPE_NONE = 0,
  USB_ERROR_TYPE_STRING,
  USB_ERROR_TYPE_ERRNO,
} usb_error_type_t;

struct usb_bus {
    struct usb_bus *    next;
    struct usb_bus *    prev;
	char                dirname[ PATH_MAX+1 ];
    struct usb_device * devices;
    uint32              location;
    struct usb_device * root_dev;
};

/* All standard descriptors have these 2 fields in common */
struct usb_descriptor_header {
    uint8   bLength;
    uint8   bDescriptorType;
};

/* String descriptor */
struct usb_string_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint16  wData[1];
};

/* HID descriptor */
struct usb_hid_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint16  bcdHID;
    uint8   bCountryCode;
    uint8   bNumDescriptors;
    /* uint8  bReportDescriptorType; */
    /* uint16 wDescriptorLength; */
    /* ... */
};

/* Endpoint descriptor */
#define USB_MAXENDPOINTS    32
struct usb_endpoint_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint8   bEndpointAddress;
    uint8   bmAttributes;
    uint16  wMaxPacketSize;
    uint8   bInterval;
    uint8   bRefresh;
    uint8   bSynchAddress;

    STRPTR  extra;   /* Extra descriptors */
    int32   extralen;
};

/* Interface descriptor */
#define USB_MAXINTERFACES   32
struct usb_interface_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint8   bInterfaceNumber;
    uint8   bAlternateSetting;
    uint8   bNumEndpoints;
    uint8   bInterfaceClass;
    uint8   bInterfaceSubClass;
    uint8   bInterfaceProtocol;
    uint8   iInterface;

    struct usb_endpoint_descriptor *endpoint;

    STRPTR  extra;   /* Extra descriptors */
    int32   extralen;
};

#define USB_MAXALTSETTING   128 /* Hard limit */

// This struct has a name thats collide with
// the Sirion stack include files

struct usb_interface {
    struct usb_interface_descriptor *altsetting;
    int32   num_altsetting;
};

/* Configuration descriptor information.. */
#define USB_MAXCONFIG       8
struct usb_config_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint16  wTotalLength;
    uint8   bNumInterfaces;
    uint8   bConfigurationValue;
    uint8   iConfiguration;
    uint8   bmAttributes;
    uint8   MaxPower;

    struct usb_interface *interface;

    STRPTR  extra;   /* Extra descriptors */
    int32   extralen;
};

/* Device descriptor */
struct usb_device_descriptor {
    uint8   bLength;
    uint8   bDescriptorType;
    uint16  bcdUSB;
    uint8   bDeviceClass;
    uint8   bDeviceSubClass;
    uint8   bDeviceProtocol;
    uint8   bMaxPacketSize0;
    uint16  idVendor;
    uint16  idProduct;
    uint16  bcdDevice;
    uint8   iManufacturer;
    uint8   iProduct;
    uint8   iSerialNumber;
    uint8   bNumConfigurations;
};

struct usb_device {
    struct usb_device *next, *prev;

	char filename[ PATH_MAX+1 ];

    struct usb_bus *bus;

    struct usb_device_descriptor    descriptor;
    struct usb_config_descriptor *  config;

    struct LibusbAmigaData *data;       // Amiga Specifik

    uint8   devnum;
    uint8   num_children;
    struct usb_device **children;
};

struct usb_ctrl_setup {
    uint8   bRequestType;
    uint8   bRequest;
    uint16  wValue;
    uint16  wIndex;
    uint16  wLength;
};

typedef struct usb_dev_handle {
    int32                   fd;

    struct usb_bus *        bus;
    struct usb_device *     device;

    int32                   config;
    int32                   interface;
    int32                   altsetting;
    struct LibusbAmigaData *data;       // Amiga Specifik
} usb_dev_handle;

/*------------------------------------------------------------------------*/

#ifdef __GNUC__
    #ifdef __PPC__
        #pragma pack()
    #endif
#elif defined(__VBCC__)
    #pragma default-align
#endif

/*------------------------------------------------------------------------*/

#ifdef __cplusplus
}
#endif

/*------------------------------------------------------------------------*/

#endif /* LIBRARIES_LIBUSB_H */

